#!/usr/bin/perl
# mb2gpsdrive 0.0.10-pre2	fetches mapblast directions and builds a 
# 				gpsdrive waypoint file
#
# The latest version of mb2gpsdrive can be found at
# http://spoiledmeat.net/projects/mb2gpsdrive/
# 
# Copyleft 2002 by Sven Fichtner <sven.fichtner@flugfunk.de>
# 
# Some code is stolen from mb2route written by Doug Rinckes. Thanks Doug! ;)
# See http://opencast.mine.nu/mod.php?mod=userpage&menu=9&page_id=4 for
# further information about mb2route
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# changelog:
# who				when		what
# ############################################################################
# sven.fichtner@flugfunk.de	2002-05-07	- removed the comment line from
# 						  output file as it would be
# 						  recognized as a waypoint
# sven.fichtner@flugfunk.de	2002-05-12	- got some hints from
# 						  kevin stephens about how
# 						  to fetch the data. thanks!
# sven.fichtner@flugfunk.de	2002-05-14	- added -D -A -S
# 						- screwed up mapblast
# 						  several times.. ;)
# sven.fichtner@flugfunk.de	2002-05-15	- LC_ALL-patch applied
# 						- PATHCALC SRVR Connection
# 						  timed out will be reported
# 						  to the user when mapblast
# 						  ends up with this error.
# sven.fichtner@flugfunk.de	2002-06-05	- Fritz' "s/./\\\_/g" patch
# 						  applied.
# sven.fichtner@flugfunk.de	2002-07-14	- verbose mode added
# 						- interactive mode added
# 						- changed the default output
# 						  file
# sven.fichtner@flugfunk.de	2002-07-24	- added a country list within 
# 						  the interactive mode
# sven.fichtner@flugfunk.de	2002-07-27	- added reversed routes
# 						- cleaned up the code. it
# 						  smells wonderful, huh?
# 						- removed -o, it became useless
# 						- wget pretends to be an IE
# 						- formfilled.html will now
# 						  be written to ~/.gpsdrive/
# 						  instead of /var/tmp/
#
# TODO:
# - clean up the code.
# - add extra hops
# - add alternate routes

# Fritz reported problems when LC_ALL is set to de_DE
system("set LC_ALL=en_US");

use Getopt::Std;
getopts('vihn:d:s:a:D:A:S:r');

# help section
if ($opt_h eq 1) {
	printf("mb2gpsdrive.pl:         uses mapblast directions to build\n");
	printf("                        a gpsdrive waypoint file\n\n");
	printf("Please see README.mb2gpsrive for a full describtion.\n\n");
	printf("  -h		        the help screen..you just found it.\n");
	printf("  -d <departure city>	Start your route here..\n");
	printf("  -D <departure street>	Street name of the first waypoint\n");
	printf("  -a <arrival city>	..and finish it there.\n");
	printf("  -A <arrival street>	Street name of the last waypoint\n");
	printf("  -s <country code>	See below and README.mb2gpsdrive\n");
	printf("  -S <country code>	If the city specified with -a is in a different\n");
	printf("			country than -s, enter it here.\n");
	printf("  -n <route name>	Prefix all waypoints with this value (prevents\n");
	printf("			waypoints overwriting each other)\n");
	printf("  -v                    verbose mode - it's noisy!\n");
	printf("  -i			interactive mode\n");
	printf("  -r			generates an additional reversed\n");
	printf("			waypoint file.\n");
	printf("\n");
	printf("Those are some possible country codes (case DOES matter)\n");
	printf("See README.mb2gpsdrive for all country codes.\n\n");
	printf("United States - US\n");
	printf("Germany - DEU\n");
	printf("Austria - AUT\n\n");
	exit;
}

# interactive mode
if ($opt_i eq 1) {
printf("route name: ");
$opt_n = <STDIN>;
printf("start city: ");
$opt_d = <STDIN>;
printf("start street: ");
$opt_D = <STDIN>;
printf("start country: (press L for a list of valid country codes) ");
$opt_s = <STDIN>;
chomp ($opt_s);
if ($opt_s eq L) {
	printf("Country codes:\n");
	printf("Albania - ALB\t\t\tArmenia - ARM\t\tAustria - AUT\n");
	printf("Azerbaijan - AZE\t\tBelarus - BLR\t\tBelguim - BEL\n");
	printf("Bosnia/Herzegovina - BIH\tBulgaria - BGR\t\tCanada - CAN\n");
	printf("Croatia - HRV\t\t\tCyprus - CYP\t\tCzech Republic - CZE\n");
	printf("Denmark - DNK\t\t\tEstonia - EST\t\tFinland - FIN\n");
	printf("France - FRA\t\t\tHungary - HUN\t\tGeorgia - GEO\n");
	printf("Germany - DEU\t\t\tGreece - GRC\t\tIreland - IRL\n");
	printf("Italy - ITA\t\t\tLatvia - LVA\t\tLiechtenstein - LIE\n");
	printf("Lithunia - LTU\t\t\tLuxembourg - LUX\tMacedonia - MKD\n");
	printf("Moldova - MDA\t\t\tNetherlands - NLD\tNorway - NOR\n");
	printf("Poland - POL\t\t\tPortugal - PRT\t\tRomania - ROM\n");
	printf("Russia - RUS\t\t\tSlovak Republic - SVK\tSlovenia - SVN\n");
	printf("Spain - ESP\t\t\tSweden - SWE\t\tSwitzerland - CHE\n");
	printf("Turkey - TUR\t\t\tUnited Kingdom - GBR\tUnited States - US\n");
	printf("Yugoslavia - YUG\n\n");
	printf("start country: ");
	$opt_s = <STDIN>;
}
printf("end city: ");
$opt_a = <STDIN>;
printf("end street: ");
$opt_A = <STDIN>;
printf("end country: ");
$opt_S = <STDIN>;
chomp ($opt_n,$opt_d,$opt_D,$opt_s,$opt_a,$opt_A,$opt_S);
}
# now assign all the variables from the parameters
if ($opt_n eq "") {
	die "You must supply a route name. (hint: -n)";
} else {
	$routenum=$opt_n;
}
if ($opt_d eq "") {
	die "You must supply the city where you want to start. (hint: -d)";
} else {
	$startcity=$opt_d;
}
if ($opt_a eq "") {
	die "Where do you want to go today? [tm] (hint: -a)";
} else {
	$endcity=$opt_a;
}
if ($opt_s eq "") {
	die "You must enter the state where you want to start";
} else {
	$startstate=$opt_s;
}
if ($opt_S eq "") {
	$endstate=$opt_s;
} else {
	$endstate=$opt_S;
}

$startstreet=$opt_D;
$endstreet=$opt_A;

# verbose mode
if ($opt_v eq 1) {
printf("route name:	$routenum\n");
printf("start city:	$startcity\n");
printf("start street:	$startstreet\n");
printf("start country:	$startstate\n");
printf("end city:	$endcity\n");
printf("end street:	$endstreet\n");
printf("end country:	$endstate\n\n");
printf("fetching http://www.mapblast.com/myblast/driveSilo.mb?remLoc_1=&AD4_1=$startstate&AD2_street_1=$startstreet&AD3_1=$startcity&apmenu_1=&apcode_1=&GAD1_1=&GAD2_1=&GAD3_1=&GAD4_1=&phone_1=&IC_1=&CT_1=&GMI_1=&remLoc_2=&AD4_2=$endstate&AD2_street_2=$endstreet&AD3_2=$endcity&apmenu_2=&apcode_2=&ndrive=1&GAD1_2=&GAD2_2=&GAD3_2=&GAD4_2=&phone_2=&IC_2=&CT_2=&GMI_2=&locBoxNum=&req_action=getdir&AD2=&AD3=&AD4=&formnum=2&forceAddr=&CT=&GMI=&multiIC=&locn_ckieVal=&serch=&skip=&CMD=FILL\n\n");
printf("hold on a second..or get another coffee\n\n");
}

# fill out mapblasts form
open(WGET,"wget -U \"Mozilla/4.0 (compatible; MSIE 5.01; Windows NT)\" -O /dev/null -v \"http://www.mapblast.com/myblast/driveSilo.mb?remLoc_1=&AD4_1=$startstate&AD2_street_1=$startstreet&AD3_1=$startcity&apmenu_1=&apcode_1=&GAD1_1=&GAD2_1=&GAD3_1=&GAD4_1=&phone_1=&IC_1=&CT_1=&GMI_1=&remLoc_2=&AD4_2=$endstate&AD2_street_2=$endstreet&AD3_2=$endcity&apmenu_2=&apcode_2=&ndrive=1&GAD1_2=&GAD2_2=&GAD3_2=&GAD4_2=&phone_2=&IC_2=&CT_2=&GMI_2=&locBoxNum=&req_action=getdir&AD2=&AD3=&AD4=&formnum=2&forceAddr=&CT=&GMI=&multiIC=&locn_ckieVal=&serch=&skip=&CMD=FILL\" 2>&1 |")  or die;

while (<WGET>) {
	next unless (/^Location/);
	$_ =~ /\s(http[^\s]+)\s/;
	$URL = $1;
}

close(WGET);
$URL .= '&showman=1';

if ($opt_v eq 1) {
	print("switched to step-by-step maps\n\n");
	print("fetching $URL\n\n");
}

# get the page showing step-by-step maps
system("wget -U \"Mozilla/4.0 (compatible; MSIE 5.01; Windows NT)\" -q -O ~/.gpsdrive/formfilled.html \"$URL\"");

# check we can open the files
open(HTML,"$ENV{'HOME'}/.gpsdrive/formfilled.html") || die "open ~/.gpsdrive/formfilled.html failed";
printf("trying to open ~/.gpsdrive/way\_$opt_n.txt\n");
open(ROUTE,">$ENV{'HOME'}/.gpsdrive/way\_$opt_n.txt") || die "open $ENV{'HOME'}/.gpsdrive/way\_$opt_n.txt failed";
$outfile = "$ENV{'HOME'}/.gpsdrive/way\_$opt_n.txt";
$waypoint=0;

# for each line in the html output...
while (<HTML>) {
	chop($_);
	# if it is a waypoint...
	if (/<a href="\/myblast\/map.mb\?\&CT=.*IC=([+-]?\d+\.\d+)\:([+-]?\d+\.\d+)\:\d+\:\d+\:([+-]?\d+\.\d+)\:([+-]?\d+\.\d+)/) {
		# get the position of this waypoint ($1 and $2) and the next one ($3 and $4)
		$north = $1; $east = $2; $north1 = $3; $east1 = $4;
		# put this waypoint into the route file
		printf(ROUTE "$routenum\_$waypoint $north $east\n");
		$waypoint ++;
	} elsif (/PATHCALC SRVR Connection timed out/) {
		print("\nMapBlast! reported an error:\n");
		print("PATHCALC SRVR Connection timed out\n\n");
		$mberror=1;
	}
}

# put the last waypoint into the route file
printf(ROUTE "$routenum\_$waypoint $north1 $east1\n");
close(HTML);
close(ROUTE);

if ($mberror eq 1) {
	print("Please rerun mb2gpsdrive");
	exit;
}

if ($waypoint eq 0) {
	die "No waypoints found in HTML file";
}

# reverse the route. we wanna find the way back home..
if ($opt_r eq 1) {
	open(REV,">$ENV{'HOME'}/.gpsdrive/way\_$opt_n\_rev.txt") || die "open $ENV{'HOME'}/.gpsdrive/way\_$opt_n\_rev.txt failed";
	open(TMP,"$outfile");
	@lines = <TMP>;
	close(TMP);
	for ($i = 0; $i < @lines; $i ++) {
	  @line = split(/ /,@lines[$i]);
	  $name[$i] = $line[0];
	  $cor1[$i] = $line[1];
	  $cor2[$i] = $line[2];
	  chomp($cor1[$i]);
	  chomp($cor2[$i]);
	}
	for ($i = 0; $i < @lines; $i ++) {
	  printf(REV "$name[$i] $cor1[@lines-$i-1] $cor2[@lines-$i-1]\n");
	}
	close(REV);
}

print("you will find the waypoints in $outfile\n");

if ($opt_r eq 1) {
	print("and the reversed route in $ENV{'HOME'}/.gpsdrive/way\_$opt_n\_rev.txt\n");
}

